/* Copyright (C) |Meso|Star> 2016 (contact@meso-star.com)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>. */

#ifndef TEST_CPR_UTILS_H
#define TEST_CPR_UTILS_H

#include <rsys/mem_allocator.h>
#include <stdio.h>

struct mesh_context {
  const double* coords;
  size_t nverts;
  const size_t* indices;
  size_t ntris;
};

static void
get_pos(const size_t ivert, double pos[2], void* context)
{
  const struct mesh_context* ctx = context;
  CHK(pos != NULL);
  CHK(context != NULL);
  CHK(ctx->coords != NULL);
  CHK(ivert < ctx->nverts);
  pos[0] = ctx->coords[ivert*2 + 0];
  pos[1] = ctx->coords[ivert*2 + 1];
}

static void
get_ids(const size_t itri, size_t ids[3], void* context)
{
  const struct mesh_context* ctx = context;
  CHK(ids != NULL);
  CHK(context != NULL);
  CHK(ctx->indices != NULL);
  CHK(itri < ctx->ntris);
  ids[0] = ctx->indices[itri*3 + 0];
  ids[1] = ctx->indices[itri*3 + 1];
  ids[2] = ctx->indices[itri*3 + 2];
}

static void
check_memory_allocator(struct mem_allocator* allocator)
{
  if(MEM_ALLOCATED_SIZE(allocator)) {
    char dump[512];
    MEM_DUMP(allocator, dump, sizeof(dump)/sizeof(char));
    fprintf(stderr, "%s\n", dump);
    FATAL("Memory leaks\n");
  }
}

#endif /* TEST_CPR_UTILS_H */
